<?php
namespace Modules\Fornecimento\Modules\AutorizacaoFornecimento\Services;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class AcompanhamentoAprovacaoService {
    protected $environment;
    protected $fileServerFilesUrl;

    public function __construct($environment, $supUtilities, $newCode, $fileUploader, $fileServerFilesUrl)
    {
        $this->environment          = $environment;
        $this->supUtilities         = $supUtilities;
        $this->newCode              = $newCode;
        $this->fileUploader         = $fileUploader;
        $this->fileServerFilesUrl   = $fileServerFilesUrl;
    }
    
    public function salvaAnexoGrid($row)
    {
        try {
            $row['NMPATHANEXO'] = $this->saveXmlInFileUplouder($row['ANEXO'][0]);
            $params = [
                'NRANEXOLANCAF'  => $this->newCode->getCodeByTableName('ANEXOLANCAF', $this->environment->getNrOrg()),
                'NRAUTOFORN'     => $row['NRAUTOFORN'],
                'CDFILDEST'      => $row['CDFILDEST'],
                'CDTELA'         => 'APPROVE',
                'DTANEXOLANCAF'  => $row['DTANEXOLANCAF'],
                'NMANEXOLANCAF'  => $row['ANEXO'][0]["name"],
                'NMPATHANEXO'    => $row['NMPATHANEXO'],
                'DSANEXOLANCAF'  => isset($row['DSANEXOLANCAF']) ? $row['DSANEXOLANCAF'] : null,
                'DSINFOANEXO'    => $this->getFileInfo($row),
                'NRORG'          => $this->environment->getNrOrg(),
                'NRORGINCLUSAO'  => $this->environment->getNrOrg(),
                'CDOPERINCLUSAO' => $this->environment->getCdOperador(),
                'NRORGULTATU'    => $this->environment->getNrOrg(),
                'CDOPERULTATU'   => $this->environment->getCdOperador(),
            ];
            
            $sql = "INSERT INTO ANEXOLANCAF ( NRANEXOLANCAF, CDTELA, DTANEXOLANCAF, NMANEXOLANCAF, NMPATHANEXO, DSANEXOLANCAF, DSINFOANEXO, NRAUTOFORN, CDFILDEST, NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, DTULTATU, NRORGULTATU, CDOPERULTATU ) VALUES ( :NRANEXOLANCAF, :CDTELA, :DTANEXOLANCAF, :NMANEXOLANCAF, :NMPATHANEXO, :DSANEXOLANCAF, :DSINFOANEXO, :NRAUTOFORN, :CDFILDEST, :NRORG, SYSDATE, :NRORGINCLUSAO, :CDOPERINCLUSAO, SYSDATE, :NRORGULTATU, :CDOPERULTATU )";
            
            $std = $this->supUtilities->crud($sql, $params);
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function saveXmlInFileUplouder($arquivo)
    {
        if (isset($arquivo)) {
            
            $file = array(
                "name" => time().'.'.explode('/', $arquivo["type"])[1], 
                "lastModified" => $arquivo["lastModified"],
                "lastModifiedDate" => $arquivo["lastModifiedDate"],
                "webkitRelativePath" => $arquivo["webkitRelativePath"],
                "type" => $arquivo["type"],
                "size"  => $arquivo["size"], 
                "b64File" => $arquivo["b64File"]);
                
            $row['ANEXOLANCAF']    = array();
            $row['ANEXOLANCAF'][0] = $file;
            
            $this->fileUploader->uploadFilesInRow($row, 'ANEXOLANCAF', 'ANEXOLANCAF');
            
            return $row['ANEXOLANCAF'][0]['path'];  
        } else {
            return false;
        }
    }
    
    public function deleteAnexoGrid($rows)
    {
        try {
            foreach ($rows as $row) {
                $this->fileUploader->deleteFile($row['NMPATHANEXO']);
                
                $params = [
                    'NRANEXOLANCAF' => $row['NRANEXOLANCAF'],
                    'CDTELA'        => $row['CDTELA']
                ];
                
                $sql = "DELETE FROM ANEXOLANCAF WHERE NRANEXOLANCAF = :NRANEXOLANCAF AND CDTELA = :CDTELA";
                
                $std = $this->supUtilities->crud($sql, $params);
            }
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function getFileInfo($row)
    {
        return $row['ANEXO'][0]["lastModified"] . '_' . $row['ANEXO'][0]["lastModifiedDate"] . '_' . 
                $row['ANEXO'][0]["webkitRelativePath"]. '_' . $row['ANEXO'][0]["type"] . '_' . 
                $row['ANEXO'][0]["size"] . '_' . $row['ANEXO'][0]["name"];
    }
    
    public function getFileServerUrl() 
    {
        return $this->fileServerFilesUrl;
    }
    
    public function downloadArquivoAnexoFileServerSup($arquivo, $fileName) 
    {
        try {
            if(isset($arquivo)) {
                $contentLenghtArray = array();
                
                foreach(get_headers($arquivo) as $value) {
                    if(strpos($value, 'Content-Length: ') !== false) {
                    	array_push($contentLenghtArray, str_replace('Content-Length: ', '',$value));
                    }
                };
                
                $contentLenght = max($contentLenghtArray);
                
                $key = $this->hasErrorOnResponse(get_headers($arquivo));
                
                if(is_numeric($key)) {
                    throw new \Exception(get_headers($arquivo)[$key]);
                }
                
                $tipo = null;
    
                switch (strtolower(substr(strrchr(basename($arquivo), "."), 1))) {
                    case "gif" : $tipo="image/gif"                    ; break;
                    case "png" : $tipo="image/png"                    ; break;
                    case "jpg" : $tipo="image/jpg"                    ; break;
                    case "wma" : $tipo="audio/mime"                   ; break;
                    case "txt" : $tipo="text/plain"                   ; break;
                    case "rem" : $tipo="text/plain"                   ; break;
                    case "mp3" : $tipo="audio/mpeg"                   ; break;
                    case "mp4" : $tipo="video/mpeg-4"                 ; break;
                    case "pdf" : $tipo="application/pdf"              ; break;
                    case "zip" : $tipo="application/zip"              ; break;
                    case "doc" : $tipo="application/msword"           ; break;
                    case "xls" : $tipo="application/vnd.ms-excel"     ; break;
                    case "exe" : $tipo="application/octet-stream"     ; break;
                    case "ppt" : $tipo="application/vnd.ms-powerpoint"; break;
                    case "php" :                                      ; break;
                    case "htm" :                                      ; break;
                    case "html":                                      ; break;
                }
                
                header("Content-Type: ".$tipo);
                header("Content-Length: ".$contentLenght);
                header("Content-Disposition: attachment; filename=".basename($fileName));
                
                readfile($arquivo);
                
                die();
            } else {
                throw new \Exception('URL não definida.');
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function hasErrorOnResponse($headers)
    {
        $errors = [
            'HTTP/1.1 404 Not Found',
            'HTTP/1.1 500 Internal Server Error',
        ];
        
        foreach ($errors as $error) {
            $key = array_search($error, $headers);
            
            if (is_numeric($key)) {
                return $key;
            }
        }
        
        return false;
    }
}    